@@ -17,7 +17,6 @@ eLCM functionality.
 """
 
 import collections
-import six
 import time
 
 from oslo_serialization import jsonutils
@@ -188,6 +187,10 @@ def elcm_request(irmc_info, method, path, **kwargs):
           'irmc_port': 80 or 443, default is 443,
           'irmc_auth_method': 'basic' or 'digest', default is 'basic',
           'irmc_client_timeout': timeout, default is 60,
+          'irmc_verify_ca': Either a boolean, in which case it controls
+                            whether we verify the server's TLS certificate,
+                            or a string, in which case it must be a path to
+                            a CA bundle to use. Defaults to ``False``.
           ...
         }
     :param method: request method such as 'GET', 'POST'
@@ -203,6 +206,7 @@ def elcm_request(irmc_info, method, path, **kwargs):
     userid = irmc_info['irmc_username']
     password = irmc_info['irmc_password']
     client_timeout = irmc_info.get('irmc_client_timeout', 60)
+    verify = irmc_info.get('irmc_verify_ca', False)
 
     # Request headers, params, and data
     headers = kwargs.get('headers', {'Accept': 'application/json'})
@@ -229,7 +233,7 @@ def elcm_request(irmc_info, method, path, **kwargs):
                              headers=headers,
                              params=params,
                              data=data,
-                             verify=False,
+                             verify=verify,
                              timeout=client_timeout,
                              allow_redirects=False,
                              auth=auth_obj)
@@ -672,8 +676,12 @@ def _process_session_data(irmc_info, operation, session_id,
                 if operation == 'CONFIG_RAID':
                     return result
 
-                elcm_profile_delete(irmc_info=irmc_info,
-                                    profile_name=PROFILE_BIOS_CONFIG)
+                # FIXME: Currently, creating a profile will restart the
+                # machine, which will cause an error during IPI installation,
+                # so temporarily comment out the operation of deleting the
+                # profile.
+                # elcm_profile_delete(irmc_info=irmc_info,
+                #                    profile_name=PROFILE_BIOS_CONFIG)
 
             except scci.SCCIError as e:
                 result['warning'] = e
@@ -712,31 +720,36 @@ def backup_bios_config(irmc_info):
             'warning': <warning message if there is>
         }
     """
+    result = {}
+
     # 1. Make sure there is no BiosConfig profile in the store
     try:
         # Get the profile first, if not found, then an exception
         # will be raised.
-        elcm_profile_get(irmc_info=irmc_info,
-                         profile_name=PROFILE_BIOS_CONFIG)
+        result['bios_config'] = elcm_profile_get(
+            irmc_info=irmc_info,
+            profile_name=PROFILE_BIOS_CONFIG)
+        # FIXME: Currently, creating a profile will restart the machine,
+        # which will cause an error during IPI installation,
+        # so temporarily comment out the operation of deleting the profile.
         # Profile found, delete it
-        elcm_profile_delete(irmc_info=irmc_info,
-                            profile_name=PROFILE_BIOS_CONFIG)
+        # elcm_profile_delete(irmc_info=irmc_info,
+        #                    profile_name=PROFILE_BIOS_CONFIG)
     except ELCMProfileNotFound:
-        # Ignore this error as it's not an error in this case
-        pass
+        # 2. Send request to create a new profile for BiosConfig
+        session = elcm_profile_create(irmc_info=irmc_info,
+                                      param_path=PARAM_PATH_BIOS_CONFIG)
 
-    # 2. Send request to create a new profile for BiosConfig
-    session = elcm_profile_create(irmc_info=irmc_info,
-                                  param_path=PARAM_PATH_BIOS_CONFIG)
+        # 3. Profile creation is in progress, we monitor the session
+        session_timeout = irmc_info.get('irmc_bios_session_timeout',
+                                        BIOS_CONFIG_SESSION_TIMEOUT)
+        return _process_session_data(
+            irmc_info=irmc_info,
+            operation='BACKUP_BIOS',
+            session_id=session['Session']['Id'],
+            session_timeout=session_timeout)
 
-    # 3. Profile creation is in progress, we monitor the session
-    session_timeout = irmc_info.get('irmc_bios_session_timeout',
-                                    BIOS_CONFIG_SESSION_TIMEOUT)
-    return _process_session_data(
-        irmc_info=irmc_info,
-        operation='BACKUP_BIOS',
-        session_id=session['Session']['Id'],
-        session_timeout=session_timeout)
+    return result
 
 
 def restore_bios_config(irmc_info, bios_config):
@@ -771,18 +784,21 @@ def restore_bios_config(irmc_info, bios_config):
     # 1. Parse the bios config and create the input data
     input_data = _process_bios_config()
 
+    # FIXME: Currently, creating a profile will restart the machine,
+    # which will cause an error during IPI installation,
+    # so temporarily comment out the operation of deleting the profile.
     # 2. Make sure there is no BiosConfig profile in the store
-    try:
-        # Get the profile first, if not found, then an exception
-        # will be raised.
-        elcm_profile_get(irmc_info=irmc_info,
-                         profile_name=PROFILE_BIOS_CONFIG)
-        # Profile found, delete it
-        elcm_profile_delete(irmc_info=irmc_info,
-                            profile_name=PROFILE_BIOS_CONFIG)
-    except ELCMProfileNotFound:
-        # Ignore this error as it's not an error in this case
-        pass
+    # try:
+    #     Get the profile first, if not found, then an exception
+    #     will be raised.
+    #    elcm_profile_get(irmc_info=irmc_info,
+    #                     profile_name=PROFILE_BIOS_CONFIG)
+    #     Profile found, delete it
+    #    elcm_profile_delete(irmc_info=irmc_info,
+    #                        profile_name=PROFILE_BIOS_CONFIG)
+    # except ELCMProfileNotFound:
+    #     Ignore this error as it's not an error in this case
+    #    pass
 
     # 3. Send a request to apply the param values
     session = elcm_profile_set(irmc_info=irmc_info,
@@ -897,7 +913,7 @@ def _update_raid_input_data(target_raid_config, raid_input):
             array_info['LogicalDrives']['LogicalDrive'].append(
                 {'@Action': 'Create',
                  'RaidLevel': logical_disk['raid_level'],
-                 'InitMode': 'slow'})
+                 'InitMode': 'fast'})
             array_info['LogicalDrives']['LogicalDrive'][i].update({
                 "@Number": i})
 
@@ -919,7 +935,7 @@ def _update_raid_input_data(target_raid_config, raid_input):
                     "ArrayRef": [
                     ]
                 },
-                "InitMode": "slow"
+                "InitMode": "fast"
             }
 
             array_info['Arrays']['Array'].append(arrays)
@@ -1014,7 +1030,7 @@ def create_raid_configuration(irmc_info, target_raid_config):
     # create new RAID config.
     raid_adapter = get_raid_adapter(irmc_info)
     logical_drives = raid_adapter['Server']['HWConfigurationIrmc'][
-        'Adapters']['RAIDAdapter'][0]['LogicalDrives']
+        'Adapters']['RAIDAdapter'][0].get('LogicalDrives')
     session_timeout = irmc_info.get('irmc_raid_session_timeout',
                                     RAID_CONFIG_SESSION_TIMEOUT)
     if logical_drives is not None:
@@ -1092,8 +1108,8 @@ def set_bios_configuration(irmc_info, settings):
     if server_version:
         bios_config_data['Server']['@Version'] = server_version
     if bios_version:
-        bios_config_data['Server']['SystemConfig']['BiosConfig']['@Version'] = \
-            bios_version
+        (bios_config_data['Server']['SystemConfig']['BiosConfig']
+            ['@Version']) = bios_version
 
     configs = {}
     for setting_param in settings:
@@ -1101,7 +1117,7 @@ def set_bios_configuration(irmc_info, settings):
         setting_value = setting_param.get("value")
         # Revert-conversion from a string of True/False to boolean.
         # It will be raise failed if put "True" or "False" string value.
-        if isinstance(setting_value, six.string_types):
+        if isinstance(setting_value, str):
             if setting_value.lower() == "true":
                 setting_value = True
             elif setting_value.lower() == "false":
@@ -1136,6 +1152,6 @@ def get_bios_settings(irmc_info):
         type_config, config = BIOS_CONFIGURATION_DICTIONARY[
             setting_param].split("_")
         if config in bios_config_data.get(type_config, {}):
-            value = six.text_type(bios_config_data[type_config][config])
+            value = str(bios_config_data[type_config][config])
             settings.append({'name': setting_param, 'value': value})
     return settings
