@@ -22,7 +22,6 @@ import time
 
 import defusedxml.ElementTree as ET
 import requests
-import six
 
 from scciclient.irmc import ipmi
 from scciclient.irmc import snmp
@@ -250,8 +249,7 @@ class MetaShareType(type):
         return cls.CIFS
 
 
-@six.add_metaclass(MetaShareType)
-class ShareType(object):
+class ShareType(object, metaclass=MetaShareType):
     """"Virtual Media Share Type."""
     NFS = 0
     CIFS = 1
@@ -259,8 +257,10 @@ class ShareType(object):
 
 def get_share_type(share_type):
     """get share type."""
-    return({'nfs': ShareType.nfs,
-            'cifs': ShareType.cifs}[share_type.lower()])
+    return {
+        'nfs': ShareType.nfs,
+        'cifs': ShareType.cifs
+    }[share_type.lower()]
 
 
 def scci_cmd(host, userid, password, cmd, port=443, auth_method='basic',
