@@ -17,8 +17,6 @@ import re
 import socket
 import struct
 
-import six
-
 from scciclient.irmc import scci
 from scciclient.irmc.viom import elcm
 
@@ -44,8 +42,7 @@ _CNA_FCOE_FUNC_IDX = 2
 _CNA_ISCSI_FUNC_IDX = 3
 
 
-@six.add_metaclass(abc.ABCMeta)
-class _PortHandler(object):
+class _PortHandler(object, metaclass=abc.ABCMeta):
     """VIOM Configurator with physical port information"""
 
     def __init__(self, slot_type, card_type, slot_idx, card_idx, port_idx):
