@@ -16,8 +16,6 @@ import abc
 import json
 import time
 
-import six
-
 from scciclient.irmc import elcm
 from scciclient.irmc import scci
 
@@ -91,7 +89,8 @@ class ELCMVIOMClient(object):
     def set_profile(self, adapter_config):
         _adapter_config = dict(adapter_config)
         _adapter_config.update({'@Processing': 'execute'})
-        req = {'Server': {'AdapterConfigIrmc': _adapter_config}}
+        req = {'Server': {'AdapterConfigIrmc': _adapter_config,
+                          '@Version': '1.01'}}
         resp = elcm.elcm_profile_set(self.irmc_info, req)
         self._wait_session(resp['Session']['Id'])
 
@@ -120,8 +119,7 @@ class VIOMAttribute(object):
         self.init = init
 
 
-@six.add_metaclass(abc.ABCMeta)
-class VIOMElement(object):
+class VIOMElement(object, metaclass=abc.ABCMeta):
     """Element in VIOM table."""
     def __init__(self, **kwargs):
         for attr in self.__class__._BASIC_ATTRIBUTES:
@@ -147,7 +145,8 @@ class VIOMTable(VIOMElement):
         VIOMAttribute('boot_mode', 'BootMode'),
         VIOMAttribute('init_boot', 'InitBoot'),
         VIOMAttribute('processing', '@Processing'),
-        VIOMAttribute('mode', 'Mode')
+        VIOMAttribute('mode', 'Mode'),
+        VIOMAttribute('version', '@Version', '1.00'),
     ]
 
     def __init__(self, **kwargs):
@@ -285,8 +284,7 @@ class Slot(VIOMElement):
         return json
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PCICard(object):
+class PCICard(object, metaclass=abc.ABCMeta):
     "Abstract class for PCI cards."
     def __init__(self, card_idx, adapter):
         self.card_idx = card_idx
@@ -339,8 +337,7 @@ class AddOnCard(PCICard):
     INDEX_KEY = '@AddOnCardIdx'
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Adapter(object):
+class Adapter(object, metaclass=abc.ABCMeta):
     """Abstract class for adapters.
 
     Adapter represents type of PCI card.
@@ -393,8 +390,7 @@ class CNAAdapter(Adapter):
     ADAPTER_NAME = 'CNAAdapter'
 
 
-@six.add_metaclass(abc.ABCMeta)
-class AdapterPort(VIOMElement):
+class AdapterPort(VIOMElement, metaclass=abc.ABCMeta):
     """Port in adapters."""
 
     def __init__(self, port_idx, **kwargs):
@@ -553,8 +549,7 @@ class CNAPort(AdapterPort):
         return port
 
 
-@six.add_metaclass(abc.ABCMeta)
-class CNAFunction(VIOMElement):
+class CNAFunction(VIOMElement, metaclass=abc.ABCMeta):
     """Abstract class for Functions for CNA card"""
     _BASIC_ATTRIBUTES = [
         VIOMAttribute('function_enable', 'FunctionEnable'),
@@ -727,8 +722,7 @@ class ISCSIFunction(CNAFunction):
         return {'MAC': self.mac} if self.mac else None
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Boot(VIOMElement):
+class Boot(VIOMElement, metaclass=abc.ABCMeta):
     """Abstract class for BootProtocol"""
     _BASIC_ATTRIBUTES = []
 
