@@ -15,11 +15,14 @@
 Test class for iRMC Power Driver
 """
 
+import builtins
+import io
 import os
 import time
+from unittest import mock
 import xml.etree.ElementTree as ET
 
-import mock
+import defusedxml.ElementTree as dET
 from requests_mock.contrib import fixture as rm_fixture
 import testtools
 
@@ -28,6 +31,103 @@ from scciclient.irmc import scci
 from scciclient.irmc import snmp
 
 
+IRMC_CONFIG_PARTIAL = """<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>
+<CMDSEQ>
+<!-- "Many Items Ommited..." -->
+<!-- "ConfBmcRemoteFdImageServer" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A50" OI="0" Type="SET">
+  <DATA Type="xsd::string">10.124.196.156</DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteFdImageUserName" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A51" OI="0" Type="SET">
+  <DATA Type="xsd::string"></DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteFdImageUserPassword" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A52" OI="0" Type="SET">
+  <DATA Type="xsd::string"></DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteFdImageUserDomain" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A53" OI="0" Type="SET">
+  <DATA Type="xsd::string"></DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteFdImageShareType" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A54" OI="0" Type="SET">
+  <DATA Type="xsd::integer">1</DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteFdImageShareName" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A55" OI="0" Type="SET">
+  <DATA Type="xsd::string">/srv/nfs</DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteFdImageImageName" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A56" OI="0" Type="SET">
+  <DATA Type="xsd::string">virtual-media.img</DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcMediaOptionsFdAttachMode" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A57" OI="0" Type="SET">
+  <DATA Type="xsd::integer">1</DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcMediaOptionsFdNumber" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A58" OI="0" Type="SET">
+  <DATA Type="xsd::integer">1</DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteCdImageServer" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A60" OI="0" Type="SET">
+  <DATA Type="xsd::string">10.124.196.156</DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteCdImageUserName" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A61" OI="0" Type="SET">
+  <DATA Type="xsd::string"></DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteCdImageUserPassword" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A62" OI="0" Type="SET">
+  <DATA Type="xsd::string"></DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteCdImageUserDomain" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A63" OI="0" Type="SET">
+  <DATA Type="xsd::string"></DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteCdImageShareType" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A64" OI="0" Type="SET">
+  <DATA Type="xsd::integer">1</DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteCdImageShareName" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A65" OI="0" Type="SET">
+  <DATA Type="xsd::string">/srv/nfs</DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcRemoteCdImageImageName" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A66" OI="0" Type="SET">
+  <DATA Type="xsd::string">virtual-media.iso</DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcMediaOptionsCdAttachMode" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A67" OI="0" Type="SET">
+  <DATA Type="xsd::integer">1</DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "ConfBmcMediaOptionsCdNumber" -->
+<CMD Context="SCCI" OC="ConfigSpace" OE="1A68" OI="0" Type="SET">
+  <DATA Type="xsd::integer">1</DATA>
+  <STATUS>0</STATUS>
+</CMD>
+<!-- "Many Items Ommited..." -->
+</CMDSEQ>"""  # noqa: E501
+
+
 class SCCITestCase(testtools.TestCase):
     """Tests for SCCI
 
@@ -43,13 +143,13 @@ class SCCITestCase(testtools.TestCase):
                 os.path.dirname(__file__),
                 'fixtures/irmc_report_ok.xml'), "r") as report_ok:
             self.report_ok_txt = report_ok.read()
-        self.report_ok_xml = ET.fromstring(self.report_ok_txt)
+        self.report_ok_xml = dET.fromstring(self.report_ok_txt)
 
         with open(os.path.join(
                 os.path.dirname(__file__),
                 'fixtures/irmc_report_ng.xml'), "r") as report_ng:
             self.report_ng_txt = report_ng.read()
-        self.report_ng_xml = ET.fromstring(self.report_ng_txt)
+        self.report_ng_xml = dET.fromstring(self.report_ng_txt)
 
         self.irmc_address = '10.124.196.159'
         self.irmc_username = 'admin'
@@ -264,7 +364,8 @@ class SCCITestCase(testtools.TestCase):
             'HTTP PROTOCOL ERROR, STATUS CODE = 302',
             str(e))
 
-    def test_power_on_ok(self):
+    @mock.patch.object(time, 'sleep')
+    def test_power_on_ok(self, mock_sleep):
         self.requests_mock.post("http://" + self.irmc_address + "/config",
                                 text="""<?xml version="1.0" encoding="UTF-8"?>
                                 <Status>
@@ -281,8 +382,10 @@ class SCCITestCase(testtools.TestCase):
                                  client_timeout=self.irmc_client_timeout)
         r = client(scci.POWER_ON)
         self.assertEqual(r.status_code, 200)
+        mock_sleep.assert_called_once_with(3)
 
-    def test_power_off_ok(self):
+    @mock.patch.object(time, 'sleep')
+    def test_power_off_ok(self, mock_sleep):
         self.requests_mock.post("http://" + self.irmc_address + "/config",
                                 text="""<?xml version="1.0" encoding="UTF-8"?>
                                 <Status>
@@ -299,6 +402,7 @@ class SCCITestCase(testtools.TestCase):
                                  client_timeout=self.irmc_client_timeout)
         r = client(scci.POWER_OFF)
         self.assertEqual(r.status_code, 200)
+        mock_sleep.assert_called_once_with(3)
 
     def test_power_cycle_ok(self):
         self.requests_mock.post("http://" + self.irmc_address + "/config",
@@ -318,7 +422,8 @@ class SCCITestCase(testtools.TestCase):
         r = client(scci.POWER_CYCLE)
         self.assertEqual(r.status_code, 200)
 
-    def test_power_reset_ok(self):
+    @mock.patch.object(time, 'sleep')
+    def test_power_reset_ok(self, mock_sleep):
         self.requests_mock.post("http://" + self.irmc_address + "/config",
                                 text="""<?xml version="1.0" encoding="UTF-8"?>
                                 <Status>
@@ -335,6 +440,7 @@ class SCCITestCase(testtools.TestCase):
                                  client_timeout=self.irmc_client_timeout)
         r = client(scci.POWER_RESET)
         self.assertEqual(r.status_code, 200)
+        mock_sleep.assert_called_once_with(3)
 
     def test_power_raise_nmi_ok(self):
         self.requests_mock.post("http://" + self.irmc_address + "/config",
@@ -498,7 +604,7 @@ class SCCITestCase(testtools.TestCase):
 
     def test_get_sensor_data_records_ng(self):
         sensor = scci.get_sensor_data_records(self.report_ng_xml)
-        self.assertEqual(sensor, None)
+        self.assertIsNone(sensor)
 
     def test_get_irmc_version_ok(self):
         version = scci.get_irmc_version(self.report_ok_xml)
@@ -506,7 +612,7 @@ class SCCITestCase(testtools.TestCase):
 
     def test_get_irmc_version_ng(self):
         version = scci.get_irmc_version(self.report_ng_xml)
-        self.assertEqual(version, None)
+        self.assertIsNone(version)
 
     def test_get_report_ok(self):
         self.requests_mock.get(
@@ -651,6 +757,9 @@ class SCCITestCase(testtools.TestCase):
                                 <Message>No Error</Message>
                                 </Status>""")
 
+        self.requests_mock.get("http://" + self.irmc_address +
+                               "/iRMC_Settings.pre", text=IRMC_CONFIG_PARTIAL)
+
         client = scci.get_client(self.irmc_address,
                                  self.irmc_username,
                                  self.irmc_password,
@@ -912,3 +1021,260 @@ class SCCITestCase(testtools.TestCase):
         fgi_status_expect = {'0': 'Idle'}
         result = scci.get_raid_fgi_status(report_fake)
         self.assertEqual(result, fgi_status_expect)
+
+    @mock.patch('scciclient.irmc.scci.requests.get')
+    def test_fail_get_bios_firmware_status(self, mock_requests_get):
+        mock_requests_get.return_value = mock.Mock(
+            status_code=404,
+            text="""</head>
+            <body>
+            <div id="main">
+                <div class="content message">
+                    <div id="alert_icon">
+                    </div>
+                    <div id="msg_title" class="title">
+                        File not found  <!-- This tag is OPTIONAL and is the
+                        title eg. 404 - File not found -->
+                    </div>
+                </div>
+            </div>
+            </body>
+            </html>""")
+        upgrade_type = 'biosss'
+        self.assertRaises(scci.SCCIClientError,
+                          scci.get_firmware_upgrade_status, self.irmc_info,
+                          upgrade_type=upgrade_type)
+
+    @mock.patch('scciclient.irmc.scci.requests.get')
+    def test_success_get_bios_firmware_status(self, mock_requests_get):
+        mock_requests_get.return_value = mock.Mock(
+            return_value='ok',
+            status_code=200,
+            text="""<?xml version="1.0" encoding="UTF-8"?>
+            <Status>
+            <Value>0</Value>
+            <Severity>Information</Severity>
+            <Message>No Error</Message>
+            </Status>""")
+        expected_status = "0"
+        expected_severity = "Information"
+        expected_message = "No Error"
+        upgrade_type = 'bios'
+        result = scci.get_firmware_upgrade_status(self.irmc_info, upgrade_type)
+        self.assertEqual(expected_status, result.find("./Value").text)
+        self.assertEqual(expected_severity, result.find("./Severity").text)
+        self.assertEqual(expected_message, result.find("./Message").text)
+
+    @mock.patch('scciclient.irmc.scci.requests.get')
+    def test_fail_get_irmc_firmware_status(self, mock_requests_get):
+        mock_requests_get.return_value = mock.Mock(
+            status_code=404,
+            text="""</head>
+            <body>
+            <div id="main">
+                <div class="content message">
+                    <div id="alert_icon">
+                    </div>
+                    <div id="msg_title" class="title">
+                        File not found  <!-- This tag is OPTIONAL and is the
+                        title eg. 404 - File not found -->
+                    </div>
+                </div>
+            </div>
+            </body>
+            </html>""")
+        upgrade_type = 'irmcccc'
+        self.assertRaises(scci.SCCIClientError,
+                          scci.get_firmware_upgrade_status, self.irmc_info,
+                          upgrade_type=upgrade_type)
+
+    @mock.patch('scciclient.irmc.scci.requests.get')
+    def test_success_get_irmc_firmware_status(self, mock_requests_get):
+        mock_requests_get.return_value = mock.Mock(
+            return_value='ok',
+            status_code=200,
+            text="""<?xml version="1.0" encoding="UTF-8"?>
+            <Status>
+            <Value>0</Value>
+            <Severity>Information</Severity>
+            <Message>No Error</Message>
+            </Status>""")
+        expected_status = "0"
+        expected_severity = "Information"
+        expected_message = "No Error"
+        upgrade_type = 'irmc'
+        result = scci.get_firmware_upgrade_status(self.irmc_info, upgrade_type)
+        self.assertEqual(expected_status, result.find("./Value").text)
+        self.assertEqual(expected_severity, result.find("./Severity").text)
+        self.assertEqual(expected_message, result.find("./Message").text)
+
+    @mock.patch('scciclient.irmc.scci.get_firmware_upgrade_status')
+    def test_failed_process_session_bios_status(self, mock_status):
+        session_timeout = 180
+        upgrade_type = 'bios'
+        # Fake status from server
+        status_fake = ET.Element(self)
+        status_fake.append(ET.Element("Value", name="Value"))
+        status_fake.append(ET.Element("Severity", name="Severity"))
+        status_fake.append(ET.Element("Message", name="Message"))
+        status_fake.find("./Value").text = '0'
+        status_fake.find("./Severity").text = 'Error'
+        status_fake.find("./Message").text = 'File not provided'
+
+        mock_status.return_value = status_fake
+        expected_status = 'Error'
+        result = scci.process_session_status(self.irmc_info, session_timeout,
+                                             upgrade_type)
+        self.assertEqual(expected_status, result['upgrade_status'])
+
+    @mock.patch('scciclient.irmc.scci.get_firmware_upgrade_status')
+    def test_success_process_session_bios_status(self, mock_status):
+        session_timeout = 180
+        upgrade_type = 'bios'
+        # Fake status from server
+        status_fake = ET.Element(self)
+        status_fake.append(ET.Element("Value", name="Value"))
+        status_fake.append(ET.Element("Severity", name="Severity"))
+        status_fake.append(ET.Element("Message", name="Message"))
+        status_fake.find("./Value").text = '9'
+        status_fake.find("./Severity").text = 'Information'
+        status_fake.find("./Message").text = 'FLASH successful'
+
+        mock_status.return_value = status_fake
+        expected_status = 'Complete'
+        result = scci.process_session_status(self.irmc_info, session_timeout,
+                                             upgrade_type)
+        self.assertEqual(expected_status, result['upgrade_status'])
+
+    @mock.patch('scciclient.irmc.scci.get_firmware_upgrade_status')
+    def test_failed_process_session_irmc_status(self, mock_status):
+        session_timeout = 180
+        upgrade_type = 'irmc'
+        # Fake status from server
+        status_fake = ET.Element(self)
+        status_fake.append(ET.Element("Value", name="Value"))
+        status_fake.append(ET.Element("Severity", name="Severity"))
+        status_fake.append(ET.Element("Message", name="Message"))
+        status_fake.find("./Value").text = '0'
+        status_fake.find("./Severity").text = 'Error'
+        status_fake.find("./Message").text = 'File not provided'
+
+        mock_status.return_value = status_fake
+        expected_status = 'Error'
+        result = scci.process_session_status(self.irmc_info, session_timeout,
+                                             upgrade_type)
+        self.assertEqual(expected_status, result['upgrade_status'])
+
+    @mock.patch('scciclient.irmc.scci.get_firmware_upgrade_status')
+    def test_success_process_session_irmc_status(self, mock_status):
+        session_timeout = 180
+        upgrade_type = 'irmc'
+        # Fake status from server
+        status_fake = ET.Element(self)
+        status_fake.append(ET.Element("Value", name="Value"))
+        status_fake.append(ET.Element("Severity", name="Severity"))
+        status_fake.append(ET.Element("Message", name="Message"))
+        status_fake.find("./Value").text = '9'
+        status_fake.find("./Severity").text = 'Information'
+        status_fake.find("./Message").text = 'FLASH successful'
+
+        mock_status.return_value = status_fake
+        expected_status = 'Complete'
+        result = scci.process_session_status(self.irmc_info, session_timeout,
+                                             upgrade_type)
+        self.assertEqual(expected_status, result['upgrade_status'])
+
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test_create_bios_firmware_upgrade(self, open_mock):
+        upgrade_type = 'bios'
+        self.requests_mock.post("http://" + self.irmc_address + "/biosupdate",
+                                text="""<?xml version="1.0" encoding="UTF-8"?>
+                                <Status>
+                                    <Value>0</Value>
+                                    <Severity>Information</Severity>
+                                    <Message>No Error</Message>
+                                </Status>""")
+        bios_input = '/media/DATA/D3099-B1.UPC'
+        open_mock.return_value = mock.mock_open(read_data="file").return_value
+        client = scci.get_client(self.irmc_address,
+                                 self.irmc_username,
+                                 self.irmc_password,
+                                 port=self.irmc_port,
+                                 auth_method=self.irmc_auth_method,
+                                 client_timeout=self.irmc_client_timeout,
+                                 upgrade_type=upgrade_type)
+        r = client(bios_input)
+        self.assertEqual(r.status_code, 200)
+
+    @mock.patch.object(builtins, 'open', side_effect=IOError, autospec=True)
+    def test_create_fail_bios_firmware_upgrade(self, open_mock):
+        upgrade_type = 'bios'
+        self.requests_mock.post("http://" + self.irmc_address + "/biosupdate",
+                                text="""<?xml version="1.0" encoding="UTF-8"?>
+                                <Status>
+                                <Value>6</Value>
+                                <Severity>Error</Severity>
+                                <Message>File not provided</Message>
+                                </Status>
+                                """)
+        # Fake wrong file directory
+        bios_input = '/media/DATA/D3099-B101.UPC'
+        open_mock.return_value = mock.mock_open(read_data="file").return_value
+        client = scci.get_client(self.irmc_address,
+                                 self.irmc_username,
+                                 self.irmc_password,
+                                 port=self.irmc_port,
+                                 auth_method=self.irmc_auth_method,
+                                 client_timeout=self.irmc_client_timeout,
+                                 upgrade_type=upgrade_type)
+
+        self.assertRaises(scci.SCCIClientError, client, bios_input)
+
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test_create_irmc_firmware_upgrade(self, open_mock):
+        upgrade_type = 'irmc'
+        self.requests_mock.post("http://" + self.irmc_address +
+                                "/irmcupdate?flashSelect=255",
+                                text="""<?xml version="1.0" encoding="UTF-8"?>
+                                <Status>
+                                    <Value>0</Value>
+                                    <Severity>Information</Severity>
+                                    <Message>No Error</Message>
+                                </Status>""")
+        irmc_input = '/media/DATA/TX2540M1.bin'
+        open_mock.return_value = mock.mock_open(read_data="file").return_value
+        client = scci.get_client(self.irmc_address,
+                                 self.irmc_username,
+                                 self.irmc_password,
+                                 port=self.irmc_port,
+                                 auth_method=self.irmc_auth_method,
+                                 client_timeout=self.irmc_client_timeout,
+                                 upgrade_type=upgrade_type)
+        r = client(irmc_input)
+        self.assertEqual(r.status_code, 200)
+
+    @mock.patch.object(builtins, 'open', side_effect=IOError, autospec=True)
+    def test_create_fail_irmc_firmware_upgrade(self, open_mock):
+        upgrade_type = 'irmc'
+        self.requests_mock.post("http://" + self.irmc_address +
+                                "/irmcupdate?flashSelect=255",
+                                text="""<?xml version="1.0" encoding="UTF-8"?>
+                                <Status>
+                                <Value>6</Value>
+                                <Severity>Error</Severity>
+                                <Message>File not provided</Message>
+                                </Status>
+                                """)
+        # Fake wrong file directory
+        irmc_input = '/media/DATA/TX2540M1111.bin'
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
+        open_mock.return_value = mock_file_handle
+        client = scci.get_client(self.irmc_address,
+                                 self.irmc_username,
+                                 self.irmc_password,
+                                 port=self.irmc_port,
+                                 auth_method=self.irmc_auth_method,
+                                 client_timeout=self.irmc_client_timeout,
+                                 upgrade_type=upgrade_type)
+
+        self.assertRaises(scci.SCCIClientError, client, irmc_input)
