@@ -813,7 +813,7 @@ class PhysicalPortIDParseTestCase(testtools.TestCase):
 
     def _validate_handler(self, handler, handler_class, slot_type, card_type,
                           slot_idx, card_idx, port_idx):
-        self.assertTrue(isinstance(handler, handler_class))
+        self.assertIsInstance(handler, handler_class)
         self.assertEqual(slot_type, handler.slot_type)
         self.assertEqual(card_type, handler.card_type)
         self.assertEqual(slot_idx, handler.slot_idx)
